# Copyright(c) 2016 Nippon Telegraph and Telephone Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from openstack import exceptions as sdk_exc
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils
from oslo_utils import strutils

from masakariclient import api_versions
from masakariclient.common.i18n import _
import masakariclient.common.utils as masakariclient_utils

# Get the logger of this module
LOG = logging.getLogger(__name__)


class ListSegment(command.Lister):
    """List segments."""

    def get_parser(self, prog_name):
        parser = super(ListSegment, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            help=_('Limit the number of segments returned')
        )
        parser.add_argument(
            '--marker',
            metavar='<id>',
            help=_('Only return segments that appear after the given segment '
                   'ID')
        )
        parser.add_argument(
            '--sort',
            metavar='<key>[:<direction>]',
            help=_("Sorting option which is a string containing a list of "
                   "keys separated by commas. Each key can be optionally "
                   "appended by a sort direction (:asc or :desc). The valid "
                   "sort keys are: ['type', 'name', 'created_at', "
                   "'updated_at']")
        )
        parser.add_argument(
            '--filters',
            metavar='<"key1=value1;key2=value2...">',
            help=_("Filter parameters to apply on returned segments. "
                   "This can be specified multiple times, or once with "
                   "parameters separated by a semicolon. The valid filter "
                   "keys are: ['recovery_method', 'service_type']"),
            action='append'
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha
        columns = ['uuid', 'name', 'description', 'service_type',
                   'recovery_method']

        if masakari_client.default_microversion:
            api_version = api_versions.APIVersion(
                masakari_client.default_microversion)
            if api_version >= api_versions.APIVersion("1.2"):
                columns.append('is_enabled')

        queries = masakariclient_utils.format_sort_filter_params(parsed_args)
        segments = masakari_client.segments(**queries)
        formatters = {}
        return (
            columns,
            (utils.get_item_properties(p, columns, formatters=formatters)
             for p in segments)
        )


class ShowSegment(command.ShowOne):
    """Show segment details."""

    def get_parser(self, prog_name):
        parser = super(ShowSegment, self).get_parser(prog_name)
        parser.add_argument(
            'segment',
            metavar='<segment>',
            help='Segment to display (name or ID)',
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha
        uuid = masakariclient_utils.get_uuid_by_name(
            masakari_client, parsed_args.segment)
        return _show_segment(masakari_client, segment_uuid=uuid)


class CreateSegment(command.ShowOne):
    """Create segment."""

    def get_parser(self, prog_name):
        parser = super(CreateSegment, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of segment.')
        )
        parser.add_argument(
            'recovery_method',
            metavar='<recovery_method>',
            choices=['auto', 'reserved_host', 'auto_priority', 'rh_priority'],
            help=_('Recovery method of segment. The supported options are: '
                   'auto, reserved_host, auto_priority, rh_priority.')
        )
        parser.add_argument(
            'service_type',
            metavar='<service_type>',
            help=_('Service type of segment.')
        )
        parser.add_argument(
            '--is_enabled',
            metavar='<boolean>',
            help=_('The enabled flag of this segment. '
                   'Supported after microversion 1.2.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description of segment.')
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha
        attrs = {
            'name': parsed_args.name,
            'description': parsed_args.description,
            'recovery_method': parsed_args.recovery_method,
            'service_type': parsed_args.service_type,
        }

        if masakari_client.default_microversion:
            api_version = api_versions.APIVersion(
                masakari_client.default_microversion)
            if (api_version >= api_versions.APIVersion("1.2") and
                    parsed_args.is_enabled is not None):
                attrs['is_enabled'] = strutils.bool_from_string(
                    parsed_args.is_enabled,
                    strict=True)

        # Remove not specified keys
        attrs = masakariclient_utils.remove_unspecified_items(attrs)

        try:
            segment = masakari_client.create_segment(**attrs)
        except Exception as ex:
            LOG.debug(_("Failed to create segment: %s"), parsed_args)
            raise ex
        return _show_segment(masakari_client,
                             segment.uuid)


class UpdateSegment(command.ShowOne):
    """Update a segment."""

    def get_parser(self, prog_name):
        parser = super(UpdateSegment, self).get_parser(prog_name)
        parser.add_argument(
            'segment',
            metavar='<segment>',
            help=_('Name or ID of the segment to update.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of segment.')
        )
        parser.add_argument(
            '--recovery_method',
            metavar='<recovery_method>',
            choices=['auto', 'reserved_host', 'auto_priority', 'rh_priority'],
            help=_('Recovery method of segment. The supported options are: '
                   'auto, reserved_host, auto_priority, rh_priority')
        )
        parser.add_argument(
            '--service_type',
            metavar='<service_type>',
            help=_('Service type of segment.')
        )
        parser.add_argument(
            '--is_enabled',
            metavar='<boolean>',
            help=_('The enabled flag of this segment. '
                   'Supported after microversion 1.2.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description of segment.')
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha

        uuid = masakariclient_utils.get_uuid_by_name(
            masakari_client, parsed_args.segment)

        attrs = {
            'name': parsed_args.name,
            'description': parsed_args.description,
            'recovery_method': parsed_args.recovery_method,
            'service_type': parsed_args.service_type,
        }

        if masakari_client.default_microversion:
            api_version = api_versions.APIVersion(
                masakari_client.default_microversion)
            if (api_version >= api_versions.APIVersion("1.2") and
                    parsed_args.is_enabled is not None):
                attrs['is_enabled'] = strutils.bool_from_string(
                    parsed_args.is_enabled,
                    strict=True)

        # Remove not specified keys
        attrs = masakariclient_utils.remove_unspecified_items(attrs)

        try:
            masakari_client.update_segment(segment=uuid, **attrs)
        # Reraise. To unify exceptions with other functions.
        except sdk_exc.NotFoundException:
            LOG.debug(_("Segment is not found: %s"), parsed_args)
            raise sdk_exc.ResourceNotFound(
                _('No Segment found for %s') % uuid)
        except Exception as ex:
            LOG.debug(_("Failed to update segment: %s"), parsed_args)
            raise ex
        return _show_segment(masakari_client, uuid)


class DeleteSegment(command.Command):
    """Delete a segment(s)."""

    def get_parser(self, prog_name):
        parser = super(DeleteSegment, self).get_parser(prog_name)
        parser.add_argument(
            'segment',
            metavar='<segment>',
            nargs='+',
            help=_('Name or ID of segment(s) to delete')
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha
        for sid in parsed_args.segment:
            try:
                uuid = masakariclient_utils.get_uuid_by_name(
                    masakari_client, sid)
                masakari_client.delete_segment(uuid, False)
                print('Segment deleted: %s' % sid)
            except Exception as ex:
                print(ex)


def _show_segment(masakari_client, segment_uuid):
    try:
        segment = masakari_client.get_segment(segment_uuid)
    except sdk_exc.ResourceNotFound:
        raise exceptions.CommandError(_('Segment is not found: %s'
                                        ) % segment_uuid)

    formatters = {}
    columns = [
        'created_at',
        'updated_at',
        'uuid',
        'name',
        'description',
        'id',
        'service_type',
        'recovery_method',
    ]

    if masakari_client.default_microversion:
        api_version = api_versions.APIVersion(
            masakari_client.default_microversion)
        if api_version >= api_versions.APIVersion("1.2"):
            columns.append('is_enabled')

    return columns, utils.get_dict_properties(segment.to_dict(), columns,
                                              formatters=formatters)



# Copyright(c) 2016 Nippon Telegraph and Telephone Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_utils import uuidutils

from masakariclient.common import exception as exc
from masakariclient.common.i18n import _


def _format_parameters(params, parse_semicolon=True):
    """Reformat parameters into dict of format expected by the API."""
    if not params:
        return {}

    if parse_semicolon:
        # expect multiple invocations of --parameters but fall back to ';'
        # delimited if only one --parameters is specified
        if len(params) == 1:
            params = params[0].split(';')

    parameters = {}
    for p in params:
        try:
            (n, v) = p.split(('='), 1)
        except ValueError:
            msg = _('Malformed parameter(%s). Use the key=value format.') % p
            raise exc.CommandError(msg)

        if n not in parameters:
            parameters[n] = v
        else:
            if not isinstance(parameters[n], list):
                parameters[n] = [parameters[n]]
            parameters[n].append(v)

    return parameters


def remove_unspecified_items(attrs):
    """Remove the items that don't have any values."""
    for key, value in list(attrs.items()):
        if value is None:
            del attrs[key]
    return attrs


def format_sort_filter_params(parsed_args):
    queries = {}
    limit = parsed_args.limit
    marker = parsed_args.marker
    sort = parsed_args.sort
    if limit:
        queries['limit'] = limit
    if marker:
        queries['marker'] = marker

    sort_keys = []
    sort_dirs = []
    if sort:
        for sort_param in sort.split(','):
            sort_key, _sep, sort_dir = sort_param.partition(':')
            if not sort_dir:
                sort_dir = 'desc'
            elif sort_dir not in ('asc', 'desc'):
                raise exc.CommandError(_(
                    'Unknown sort direction: %s') % sort_dir)
            sort_keys.append(sort_key)
            sort_dirs.append(sort_dir)

        queries['sort_key'] = sort_keys
        queries['sort_dir'] = sort_dirs

    if parsed_args.filters:
        queries.update(_format_parameters(parsed_args.filters))

    return queries


def get_uuid_by_name(manager, name, segment=None):
    """Helper methods for getting uuid of segment or host by name.

    :param manager: A client manager class
    :param name: The resource we are trying to find a uuid
    :param segment: segment id, default None
    :return: The uuid of found resource
    """

    # If it cannot be found return the name.
    uuid = name
    if not uuidutils.is_uuid_like(name):
        if segment:
            items = manager.hosts(segment)
        else:
            items = manager.segments()

        for item in items:
            item_name = getattr(item, 'name')
            if item_name == name:
                uuid = getattr(item, 'uuid')
                break
    return uuid



# Copyright(c) 2022 Inspur
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock
import uuid

from osc_lib import utils

from masakariclient.osc.v1.vmove import ShowVMove
from masakariclient.tests import base

VMOVE_ID = uuid.uuid4()
NOTIFICATION_ID = uuid.uuid4()
SERVER_ID = uuid.uuid4()


class FakeNamespace(object):
    """Fake parser object."""
    def __init__(self, notification_id=None, vmove_id=None):
        super(FakeNamespace, self).__init__()
        self.notification_id = notification_id
        self.vmove_id = vmove_id


class FakeVMove(object):
    """Fake notification show detail."""
    def __init__(self,):
        super(FakeVMove, self).__init__()

    def to_dict(self):
        return {
            'created_at': '2023-01-28T14:55:27.000000',
            'uuid': VMOVE_ID,
            'notification_id': NOTIFICATION_ID,
            'server_id': SERVER_ID,
            'server_name': 'test',
            'source_host': 'host1',
            'dest_host': 'host2',
            'start_time': '2023-01-28T14:55:27.000000',
            'end_time': "2023-01-28T14:55:31.000000",
            'status': 'succeeded',
            'type': 'evacuation',
            'message': None,
            }


class BaseV1VMove(base.TestCase):
    def setUp(self):
        super(BaseV1VMove, self).setUp()

        self.app = mock.Mock()
        self.app_args = mock.Mock()
        self.client_manager = mock.Mock()
        self.app.client_manager.ha = self.client_manager
        self.columns = [
            'created_at', 'updated_at',
            'uuid', 'server_id', 'server_name',
            'source_host', 'dest_host',
            'start_time', 'end_time',
            'type', 'status', 'message'
        ]
        self.dummy_vmove = FakeVMove()


class TestV1ShowVMove(BaseV1VMove):
    def setUp(self):
        super(TestV1ShowVMove, self).setUp()
        self.show_vmove = ShowVMove(self.app, self.app_args,
                                    cmd_name='vmove show')

    @mock.patch.object(utils, 'get_dict_properties')
    def test_take_action(self, mock_get_dict_properties):
        parsed_args = FakeNamespace(notification_id=NOTIFICATION_ID,
                                    vmove_id=VMOVE_ID)
        self.app.client_manager.ha.get_vmove.return_value = self.dummy_vmove

        # show the vmove specified by uuid
        self.show_vmove.take_action(parsed_args)
        self.app.client_manager.ha.get_vmove.assert_called_once_with(
            VMOVE_ID, NOTIFICATION_ID)
        mock_get_dict_properties.assert_called_once_with(
            self.dummy_vmove.to_dict(), self.columns, formatters={})



# Copyright(c) 2022 Inspur
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from openstack import exceptions as sdk_exc
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from masakariclient.common.i18n import _
import masakariclient.common.utils as masakariclient_utils

# Get the logger of this module
LOG = logging.getLogger(__name__)


class ListVMove(command.Lister):
    """List VMoves."""

    def get_parser(self, prog_name):
        parser = super(ListVMove, self).get_parser(prog_name)
        parser.add_argument(
            'notification_id',
            metavar='<notification_id>',
            help=_('UUID of notification.')
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            help=_('Limit the number of vmoves returned')
        )
        parser.add_argument(
            '--marker',
            metavar='<id>',
            help=_('Only return vmoves that appear after the given vmove ID')
        )
        parser.add_argument(
            '--sort',
            metavar='<key>[:<direction>]',
            help=_("Sorting option which is a string containing a list of "
                   "keys separated by commas. Each key can be optionally "
                   "appended by a sort direction (:asc or :desc). The valid "
                   "sort keys are: ['type', 'status', 'start_time', "
                   "'end_time']")
        )
        parser.add_argument(
            '--filters',
            metavar='<"key1=value1;key2=value2...">',
            help=_("Filter parameters to apply on returned vmoves. "
                   "This can be specified multiple times, or once with "
                   "parameters separated by a semicolon. The valid filter "
                   "keys are: ['type', 'status'"),
            action='append'
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha

        columns = ['uuid', 'server_id', 'server_name',
                   'source_host', 'dest_host',
                   'start_time', 'end_time',
                   'type', 'status']

        queries = masakariclient_utils.format_sort_filter_params(parsed_args)
        vmoves = masakari_client.vmoves(parsed_args.notification_id,
                                        **queries)
        formatters = {}
        return (
            columns,
            (utils.get_item_properties(p, columns, formatters=formatters)
             for p in vmoves)
        )


class ShowVMove(command.ShowOne):
    """Show vmove details."""

    def get_parser(self, prog_name):
        parser = super(ShowVMove, self).get_parser(prog_name)
        parser.add_argument(
            'notification_id',
            metavar='<notification_id>',
            help=_('UUID of COMPUTE_NODE type notification.')
        )
        parser.add_argument(
            'vmove_id',
            metavar='<vmove_id>',
            help='UUID of the VMove',
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha
        return _show_vmove(masakari_client, parsed_args.notification_id,
                           parsed_args.vmove_id)


def _show_vmove(masakari_client, notification_id, vmove_id):
    try:
        vmove = masakari_client.get_vmove(vmove_id, notification_id)
    except sdk_exc.ResourceNotFound:
        raise exceptions.CommandError(_('VMove %s is not found.')
                                      % vmove_id)

    formatters = {}
    columns = [
        'created_at',
        'updated_at',
        'uuid',
        'server_id',
        'server_name',
        'source_host',
        'dest_host',
        'start_time',
        'end_time',
        'type',
        'status',
        'message'
    ]
    return columns, utils.get_dict_properties(vmove.to_dict(), columns,
                                              formatters=formatters)



# Copyright(c) 2016 Nippon Telegraph and Telephone Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from openstack import exceptions as sdk_exc
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from masakariclient.common.i18n import _
import masakariclient.common.utils as masakariclient_utils

# Get the logger of this module
LOG = logging.getLogger(__name__)


class ListHost(command.Lister):
    """List Hosts."""

    def get_parser(self, prog_name):
        parser = super(ListHost, self).get_parser(prog_name)
        parser.add_argument(
            'segment_id',
            metavar='<segment_id>',
            help=_('Name or ID of segment.')
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            help=_('Limit the number of hosts returned')
        )
        parser.add_argument(
            '--marker',
            metavar='<id>',
            help=_('Only return hosts that appear after the given host ID')
        )
        parser.add_argument(
            '--sort',
            metavar='<key>[:<direction>]',
            help=_("Sorting option which is a string containing a list of "
                   "keys separated by commas. Each key can be optionally "
                   "appended by a sort direction (:asc or :desc). The valid "
                   "sort keys are: ['type', 'name', 'created_at', "
                   "'updated_at']")
        )
        parser.add_argument(
            '--filters',
            metavar='<"key1=value1;key2=value2...">',
            help=_("Filter parameters to apply on returned hosts. "
                   "This can be specified multiple times, or once with "
                   "parameters separated by a semicolon. The valid filter "
                   "keys are: ['failover_segment_id', 'type', "
                   "'on_maintenance', 'reserved']"),
            action='append'
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha
        segment_id = masakariclient_utils.get_uuid_by_name(
            masakari_client, parsed_args.segment_id)

        columns = ['uuid', 'name', 'type', 'control_attributes', 'reserved',
                   'on_maintenance', 'failover_segment_id']

        queries = masakariclient_utils.format_sort_filter_params(parsed_args)
        hosts = masakari_client.hosts(segment_id, **queries)
        formatters = {}
        return (
            columns,
            (utils.get_item_properties(p, columns, formatters=formatters)
             for p in hosts)
        )


class ShowHost(command.ShowOne):
    """Show host details."""

    def get_parser(self, prog_name):
        parser = super(ShowHost, self).get_parser(prog_name)
        parser.add_argument(
            'segment_id',
            metavar='<segment_id>',
            help=_('Name or ID of segment.')
        )
        parser.add_argument(
            'host',
            metavar='<host>',
            help='Name or ID of the Host',
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha
        segment_id = masakariclient_utils.get_uuid_by_name(
            masakari_client, parsed_args.segment_id)
        uuid = masakariclient_utils.get_uuid_by_name(
            masakari_client,
            parsed_args.host,
            segment=segment_id)
        return _show_host(masakari_client, segment_id, uuid)


class CreateHost(command.ShowOne):
    """Create a Host."""

    def get_parser(self, prog_name):
        parser = super(CreateHost, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of host.')
        )
        parser.add_argument(
            'type',
            metavar='<type>',
            help=_('Type of host.')
        )
        parser.add_argument(
            'control_attributes',
            metavar='<control_attributes>',
            help=_('Attribute about control.')
        )
        parser.add_argument(
            'segment_id',
            metavar='<segment_id>',
            help=_('Name or ID of segment.')
        )
        parser.add_argument(
            '--reserved',
            metavar='<reserved>',
            choices=['True', 'False'],
            help=_('Host reservation. The supported options are: '
                   'True, False.')
        )
        parser.add_argument(
            '--on_maintenance',
            metavar='<on_maintenance>',
            choices=['True', 'False'],
            help=_('Maintenance status of host. The supported options are: '
                   'True, False.')
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha
        segment_id = masakariclient_utils.get_uuid_by_name(
            masakari_client, parsed_args.segment_id)
        attrs = {
            'name': parsed_args.name,
            'type': parsed_args.type,
            'control_attributes': parsed_args.control_attributes,
            'reserved': parsed_args.reserved,
            'on_maintenance': parsed_args.on_maintenance,
        }
        # Remove not specified keys
        attrs = masakariclient_utils.remove_unspecified_items(attrs)

        try:
            host = masakari_client.create_host(
                segment_id=segment_id,
                **attrs)
        except Exception as ex:
            LOG.debug(_("Failed to create segment host: %s"), parsed_args)
            raise ex
        return _show_host(masakari_client,
                          segment_id,
                          host.uuid)


class UpdateHost(command.ShowOne):
    """Update a Host."""

    def get_parser(self, prog_name):
        parser = super(UpdateHost, self).get_parser(prog_name)
        parser.add_argument(
            'segment_id',
            metavar='<segment_id>',
            help=_('Name or ID of segment.')
        )
        parser.add_argument(
            'host',
            metavar='<host>',
            help='Name or ID of the Host',
        )
        parser.add_argument(
            '--reserved',
            metavar='<reserved>',
            choices=['True', 'False'],
            help=_('Host reservation. The supported options are: '
                   'True, False.')
        )
        parser.add_argument(
            '--on_maintenance',
            metavar='<on_maintenance>',
            choices=['True', 'False'],
            help=_('Maintenance status of host. The supported options are: '
                   'True, False.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of host.')
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            help=_('Type of host.')
        )
        parser.add_argument(
            '--control_attributes',
            metavar='<control_attributes>',
            help=_('Attributes about control.')
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha
        segment_id = masakariclient_utils.get_uuid_by_name(
            masakari_client, parsed_args.segment_id)
        uuid = masakariclient_utils.get_uuid_by_name(
            masakari_client,
            parsed_args.host,
            segment=segment_id)
        attrs = {
            'name': parsed_args.name,
            'type': parsed_args.type,
            'control_attributes': parsed_args.control_attributes,
            'reserved': parsed_args.reserved,
            'on_maintenance': parsed_args.on_maintenance,
        }
        # Remove not specified keys
        attrs = masakariclient_utils.remove_unspecified_items(attrs)

        try:
            masakari_client.update_host(
                uuid, segment_id=segment_id, **attrs)
        except sdk_exc.NotFoundException:
            # Reraise. To unify exceptions with other functions.
            LOG.debug(_("Segment host is not found: %s"), parsed_args)
            raise sdk_exc.ResourceNotFound(
                _('No Host found for %s') % uuid)
        except Exception as ex:
            LOG.debug(_("Failed to update segment host: %s"), parsed_args)
            raise ex

        return _show_host(masakari_client, segment_id, uuid)


class DeleteHost(command.Command):
    """Delete a host."""

    def get_parser(self, prog_name):
        parser = super(DeleteHost, self).get_parser(prog_name)
        parser.add_argument(
            'segment_id',
            metavar='<segment_id>',
            help=_('Name or ID of segment.')
        )
        parser.add_argument(
            'host',
            metavar='<host>',
            help='Name or ID of the Host',
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha
        segment_id = masakariclient_utils.get_uuid_by_name(
            masakari_client, parsed_args.segment_id)
        uuid = masakariclient_utils.get_uuid_by_name(
            masakari_client,
            parsed_args.host,
            segment=segment_id)
        masakari_client.delete_host(
            uuid, segment_id=segment_id, ignore_missing=False)
        print('Host deleted: %s' % parsed_args.host)


def _show_host(masakari_client, segment_id, uuid):
    try:
        host = masakari_client.get_host(uuid, segment_id=segment_id)
    except sdk_exc.ResourceNotFound:
        raise exceptions.CommandError(_('Segment host is not found: %s'
                                        ) % uuid)

    formatters = {}
    columns = [
        'created_at',
        'updated_at',
        'uuid',
        'name',
        'type',
        'control_attributes',
        'reserved',
        'on_maintenance',
        'failover_segment_id',
    ]
    return columns, utils.get_dict_properties(host.to_dict(), columns,
                                              formatters=formatters)



# Copyright(c) 2016 Nippon Telegraph and Telephone Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import oslo_i18n

_translators = oslo_i18n.TranslatorFactory(domain='masakariclient')

# The primary translation function using the well-known name "_"
_ = _translators.primary

# Translators for log levels.
# The abbreviated names are meant to reflect the usual use of a short
# name like '_'. The "L" is for "log" and the other letter comes from
# the level.
_LI = _translators.log_info
_LW = _translators.log_warning
_LE = _translators.log_error
_LC = _translators.log_critical



# Copyright(c) 2016 Nippon Telegraph and Telephone Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
test_masakariclient
----------------------------------

Tests for `masakariclient` module.
"""
from unittest import mock

import ddt
import uuid

from osc_lib.tests import utils as osc_lib_utils
from osc_lib import utils

from masakariclient.osc.v1.segment import CreateSegment
from masakariclient.osc.v1.segment import DeleteSegment
from masakariclient.osc.v1.segment import ShowSegment
from masakariclient.osc.v1.segment import UpdateSegment
from masakariclient.tests import base

SEGMENT_NAME = 'segment_name'
SEGMENT_ID = uuid.uuid4()


class FakeNamespace(object):
    """Fake parser object."""
    def __init__(self, segment=None, name=None,
                 description=None,
                 recovery_method=None, service_type=None):
        super(FakeNamespace, self).__init__()
        self.segment = segment
        self.name = name
        self.description = description
        self.recovery_method = recovery_method
        self.service_type = service_type


class FakeSegments(object):
    """Fake segment list."""
    def __init__(self, name=None, uuid=None,
                 description=None,
                 recovery_method=None, service_type=None):
        super(FakeSegments, self).__init__()
        self.name = name
        self.uuid = uuid
        self.description = description
        self.recovery_method = recovery_method
        self.service_type = service_type


class FakeSegment(object):
    """Fake segment show detail."""
    def __init__(self,):
        super(FakeSegment, self).__init__()

    def to_dict(self):
        return {
            'created_at': '2016-12-18T05:47:46.000000',
            'updated_at': '2016-12-18T06:05:16.000000',
            'uuid': SEGMENT_ID,
            'name': SEGMENT_NAME,
            'description': 'test_segment_description',
            'id': 1,
            'service_type': 'test_type',
            'recovery_method': 'auto',
            }


class BaseV1Segment(base.TestCase):
    def setUp(self):
        super(BaseV1Segment, self).setUp()

        self.app = mock.Mock()
        self.app_args = mock.Mock()
        self.client_manager = mock.Mock()
        self.client_manager.default_microversion = '1.0'
        self.app.client_manager.ha = self.client_manager
        # segment data setup
        self.dummy_segment = FakeSegment()


class TestV1ShowSegment(BaseV1Segment):
    def setUp(self):
        super(TestV1ShowSegment, self).setUp()

        self.show_seg = ShowSegment(self.app,
                                    self.app_args,
                                    cmd_name='segment show')
        self.columns = ['created_at', 'updated_at', 'uuid',
                        'name', 'description', 'id', 'service_type',
                        'recovery_method',
                        ]
        # return value segment list
        self.dummy_segments = [FakeSegments(name=SEGMENT_NAME,
                                            uuid=SEGMENT_ID)]

    def test_take_action_by_uuid(self):

        # command param
        parsed_args = FakeNamespace(segment=SEGMENT_ID)
        self._test_take_action(parsed_args)

    def test_take_action_by_name(self):

        # command param
        parsed_args = FakeNamespace(segment=SEGMENT_NAME)
        self._test_take_action(parsed_args)

    @mock.patch.object(utils, 'get_dict_properties')
    def _test_take_action(self, parsed_args, mock_get_dict_properties):
        # return value segment list
        self.app.client_manager.ha.segments.return_value = self.dummy_segments
        # return value segment show
        self.app.client_manager.ha.get_segment.return_value = (
            self.dummy_segment)
        # show segment
        self.show_seg.take_action(parsed_args)
        mock_get_dict_properties.assert_called_once_with(
            self.dummy_segment.to_dict(), self.columns, formatters={})


class TestV1UpdateSegment(BaseV1Segment, osc_lib_utils.TestCommand):
    def setUp(self):
        super(TestV1UpdateSegment, self).setUp()

        self.update_seg = UpdateSegment(self.app,
                                        self.app_args,
                                        cmd_name='segment update')
        self.columns = ['created_at', 'updated_at', 'uuid',
                        'name', 'description', 'id', 'service_type',
                        'recovery_method',
                        ]
        # segment list
        self.dummy_segments = [
            FakeSegments(
                name=SEGMENT_NAME, uuid=SEGMENT_ID,
                description='FakeNamespace_description',
                recovery_method='Update_recovery_method',
                service_type='test_type')]
        self.dummy_segments.append(FakeSegments(
            name=SEGMENT_NAME, uuid=SEGMENT_ID,
            description='FakeNamespace_description',
            recovery_method='Update_recovery_method',
            service_type='test_type'))

    def test_take_action_by_uuid(self):

        # command param
        arglist = ['8c35987c-f416-46ca-be37-52f58fd8d294',
                   '--name', 'test_segment',
                   '--recovery_method', 'rh_priority',
                   '--service_type', 'test_service',
                   '--description', 'test_segment']

        parsed_args = self.check_parser(self.update_seg, arglist, [])
        self._test_take_action(parsed_args)

    def test_take_action_by_name(self):

        # command param
        arglist = [SEGMENT_NAME, '--name', 'test_segment',
                                 '--recovery_method', 'auto_priority',
                                 '--service_type', 'test_service',
                                 '--description', 'test_segment']

        parsed_args = self.check_parser(self.update_seg, arglist, [])
        self._test_take_action(parsed_args)

    def test_update_segment_with_recovery_method_reserved_host(self):

        arglist = ['8c35987c-f416-46ca-be37-52f58fd8d294',
                   '--name', 'test_segment',
                   '--recovery_method', 'reserved_host',
                   '--service_type', 'test_service',
                   '--description', 'test_segment']

        parsed_args = self.check_parser(self.update_seg, arglist, [])
        self._test_take_action(parsed_args)

    @mock.patch.object(utils, 'get_dict_properties')
    def _test_take_action(self, parsed_args, mock_get_dict_properties):
        # return value segment list
        self.app.client_manager.ha.segments.return_value = self.dummy_segments
        # return value segment data setup
        self.app.client_manager.ha.get_segment.return_value = (
            self.dummy_segment)
        # segment update
        self.update_seg.take_action(parsed_args)
        mock_get_dict_properties.assert_called_once_with(
            self.dummy_segment.to_dict(), self.columns, formatters={})

    def test_update_with_invalid_recovery_method(self):
        arglist = [SEGMENT_NAME, '--name', 'test_segment',
                   '--recovery_method', 'invalid-rcovery-method',
                   '--service_type', 'test_service',
                   '--description', 'test_segment']
        self.assertRaises(osc_lib_utils.ParserException,
                          self.check_parser, self.update_seg, arglist, [])


class TestV1DeleteSegment(BaseV1Segment):
    def setUp(self):
        super(TestV1DeleteSegment, self).setUp()

        self.delete_seg = DeleteSegment(self.app,
                                        self.app_args,
                                        cmd_name='segment delete')
        # segment list
        self.dummy_segments = [
            FakeSegments(
                name=SEGMENT_NAME, uuid=SEGMENT_ID,
                description='FakeNamespace_description',
                recovery_method='Update_recovery_method',
                service_type='test_type')]

    def test_take_action_by_uuid(self):

        # command param
        parsed_args = FakeNamespace(segment=[SEGMENT_ID])
        self._test_take_action(parsed_args)

    def test_take_action_by_name(self):

        # command param
        parsed_args = FakeNamespace(segment=[SEGMENT_NAME])
        self._test_take_action(parsed_args)

    def _test_take_action(self, parsed_args):

        # return_value segment list
        self.app.client_manager.ha.segments.return_value = self.dummy_segments

        # return_value segment delete
        self.app.client_manager.ha.delete_segment.return_value = None

        # segment delete
        self.delete_seg.take_action(parsed_args)

        self.app.client_manager.ha.delete_segment.assert_called_once_with(
            SEGMENT_ID, False)


@ddt.ddt
class TestV1CreateSegment(BaseV1Segment, osc_lib_utils.TestCommand):

    def setUp(self):
        super(TestV1CreateSegment, self).setUp()
        self.cmd = CreateSegment(self.app, None)

    @ddt.data({"recovery_method": "auto"},
              {"recovery_method": "reserved_host"},
              {"recovery_method": "auto_priority"},
              {"recovery_method": "rh_priority"})
    def test_create_with_all_recovery_methods(self, ddt_data):
        arglist = ['test_segment', ddt_data['recovery_method'], 'test_service',
                   '--description', 'test_segment']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self._test_take_action(parsed_args, arglist)

    def _test_take_action(self, parsed_args, arglist):
        # return value segment list
        self.app.client_manager.ha.segments.return_value = arglist
        # return value segment data setup
        self.app.client_manager.ha.get_segment.return_value = (
            self.dummy_segment)
        self.cmd.take_action(parsed_args)
        self.app.client_manager.ha.create_segment.assert_called_with(
            description='test_segment',
            name='test_segment',
            recovery_method=arglist[1],
            service_type='test_service')

    def test_create_segment_recovery_method_invalid(self):
        arglist = ['test_segment', 'invalid_recovery_method', 'test_service',
                   '--description', 'test_segment']
        self.assertRaises(osc_lib_utils.ParserException,
                          self.check_parser, self.cmd, arglist, [])



# Copyright(c) 2016 Nippon Telegraph and Telephone Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstack import exceptions as sdk_exc
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils
from oslo_serialization import jsonutils

from masakariclient import api_versions
from masakariclient.common.i18n import _
import masakariclient.common.utils as masakariclient_utils


class ListNotification(command.Lister):
    """List notifications."""

    def get_parser(self, prog_name):
        parser = super(ListNotification, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            help=_('Limit the number of notifications returned')
        )
        parser.add_argument(
            '--marker',
            metavar='<id>',
            help=_('Only return notifications that appear after the given '
                   'notification ID')
        )
        parser.add_argument(
            '--sort',
            metavar='<key>[:<direction>]',
            help=_("Sorting option which is a string containing a list of "
                   "keys separated by commas. Each key can be optionally "
                   "appended by a sort direction (:asc or :desc). The valid "
                   "sort keys are: ['type', 'created_at', 'updated_at']")
        )
        parser.add_argument(
            '--filters',
            metavar='<"key1=value1;key2=value2...">',
            help=_("Filter parameters to apply on returned notifications. "
                   "This can be specified multiple times, or once with "
                   "parameters separated by a semicolon. The valid filter "
                   "keys are: ['source_host_uuid', 'type', 'status', "
                   "generated-since]"),
            action='append'
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha
        columns = ['notification_uuid', 'generated_time', 'status',
                   'type', 'source_host_uuid', 'payload']

        queries = masakariclient_utils.format_sort_filter_params(parsed_args)
        notifications = masakari_client.notifications(**queries)
        formatters = {}
        return (
            columns,
            (utils.get_item_properties(p, columns, formatters=formatters)
             for p in notifications)
        )


class ShowNotification(command.ShowOne):
    """Show notification details."""

    def get_parser(self, prog_name):
        parser = super(ShowNotification, self).get_parser(prog_name)
        parser.add_argument(
            'notification',
            metavar='<notification>',
            help='UUID of notification to display',
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha
        return _show_notification(masakari_client,
                                  notification_uuid=parsed_args.notification)


class CreateNotification(command.ShowOne):
    """Create notification."""

    def get_parser(self, prog_name):
        parser = super(CreateNotification, self).get_parser(prog_name)
        parser.add_argument(
            'type',
            metavar='<type>',
            choices=['COMPUTE_HOST', 'VM', 'PROCESS'],
            help=_('Type of failure. The supported options are: '
                   'COMPUTE_HOST, VM, PROCESS.')
        )
        parser.add_argument(
            'hostname',
            metavar='<hostname>',
            help=_('Hostname of notification.')
        )
        parser.add_argument(
            'generated_time',
            metavar='<generated_time>',
            help=_('Timestamp for notification. e.g. 2016-01-01T01:00:00.000')
        )
        parser.add_argument(
            'payload',
            metavar='<payload>',
            help=_('JSON string about failure event.')
        )
        return parser

    def take_action(self, parsed_args):
        masakari_client = self.app.client_manager.ha
        payload = jsonutils.loads(parsed_args.payload)
        attrs = {
            'type': parsed_args.type,
            'hostname': parsed_args.hostname,
            'generated_time': parsed_args.generated_time,
            'payload': payload,
        }

        notification = masakari_client.create_notification(**attrs)
        return _show_notification(masakari_client,
                                  notification.notification_uuid)


def _show_notification(masakari_client, notification_uuid):
    try:
        notification = masakari_client.get_notification(notification_uuid)
    except sdk_exc.ResourceNotFound:
        raise exceptions.CommandError(_('Notification not found: %s'
                                        ) % notification_uuid)

    formatters = {}
    columns = [
        'created_at',
        'updated_at',
        'notification_uuid',
        'type',
        'status',
        'source_host_uuid',
        'generated_time',
        'payload'
    ]

    if masakari_client.default_microversion:
        api_version = api_versions.APIVersion(
            masakari_client.default_microversion)
        if api_version >= api_versions.APIVersion("1.1"):
            columns.append('recovery_workflow_details')

    return columns, utils.get_dict_properties(notification.to_dict(), columns,
                                              formatters=formatters)



# Copyright(c) 2016 Nippon Telegraph and Telephone Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
test_masakariclient
----------------------------------

Tests for `masakariclient` module.
"""
from unittest import mock
import uuid

from osc_lib import utils

from masakariclient.osc.v1.host import DeleteHost
from masakariclient.osc.v1.host import ShowHost
from masakariclient.osc.v1.host import UpdateHost
from masakariclient.tests import base

HOST_NAME = 'host_name'
HOST_ID = uuid.uuid4()
SEGMENT_NAME = 'segment_name'
SEGMENT_ID = uuid.uuid4()


class FakeNamespace(object):
    """Fake parser object."""
    def __init__(self, segment_id=None, host=None,
                 reserved=None, name=None, type=None,
                 control_attributes=None, on_maintenance=None):
        super(FakeNamespace, self).__init__()
        self.segment_id = segment_id
        self.host = host
        self.reserved = reserved
        self.name = name
        self.type = type
        self.control_attributes = control_attributes
        self.on_maintenance = on_maintenance


class FakeHosts(object):
    """Fake segment host list."""
    def __init__(self, name=None, uuid=None):
        super(FakeHosts, self).__init__()
        self.name = name
        self.uuid = uuid


class FakeSegments(object):
    """Fake segment show detail."""
    def __init__(self, name=None, uuid=None):
        super(FakeSegments, self).__init__()
        self.name = name
        self.uuid = uuid


class FakeHost(object):
    """Fake segment show detail."""
    def __init__(self,):
        super(FakeHost, self).__init__()

    def to_dict(self):
        return {
            'reserved': 'False',
            'uuid': HOST_ID,
            'on_maintenance': False,
            'created_at': '2016-12-18T05:47:55.000000',
            'control_attributes': 'control_attributes',
            'updated_at': None,
            'name': HOST_NAME,
            'type': 'auto',
            'id': 18,
            'failover_segment_id': SEGMENT_ID,
            }


class BaseV1Host(base.TestCase):
    def setUp(self):
        super(BaseV1Host, self).setUp()

        self.app = mock.Mock()
        self.app_args = mock.Mock()
        self.client_manager = mock.Mock()
        self.app.client_manager.ha = self.client_manager
        self.columns = [
            'created_at', 'updated_at', 'uuid', 'name', 'type',
            'control_attributes', 'reserved', 'on_maintenance',
            'failover_segment_id',
        ]
        # fake host list
        self.dummy_hosts = [FakeHosts(name=HOST_NAME, uuid=HOST_ID)]
        # fake segment list
        self.dummy_segments = [FakeSegments(name=SEGMENT_NAME,
                                            uuid=SEGMENT_ID)]
        # fake host show
        self.dummy_host = FakeHost()


class TestV1ShowHost(BaseV1Host):
    def setUp(self):
        super(TestV1ShowHost, self).setUp()
        self.show_host = ShowHost(self.app, self.app_args,
                                  cmd_name='host show')

    def test_take_action_by_uuid(self):

        # command param
        parsed_args = FakeNamespace(segment_id=SEGMENT_ID, host=HOST_ID)
        self._test_take_action(parsed_args)

    def test_take_action_by_name(self):

        # command param
        parsed_args = FakeNamespace(segment_id=SEGMENT_ID, host=HOST_NAME)
        self._test_take_action(parsed_args)

    @mock.patch.object(utils, 'get_dict_properties')
    def _test_take_action(self, parsed_args, mock_get_dict_properties):
        # return value segment list
        self.app.client_manager.ha.segments.return_value = self.dummy_segments
        # return value host list
        self.app.client_manager.ha.hosts.return_value = self.dummy_hosts
        # return value host show
        self.app.client_manager.ha.get_host.return_value = self.dummy_host

        # show the host specified by uuid
        self.show_host.take_action(parsed_args)
        self.app.client_manager.ha.get_host.assert_called_once_with(
            HOST_ID, segment_id=SEGMENT_ID)
        mock_get_dict_properties.assert_called_once_with(
            self.dummy_host.to_dict(), self.columns, formatters={})


class TestV1UpdateHost(BaseV1Host):
    def setUp(self):
        super(TestV1UpdateHost, self).setUp()
        self.update_host = UpdateHost(self.app, self.app_args,
                                      cmd_name='host update')

    def test_take_action_by_uuid(self):

        # command param
        parsed_args = FakeNamespace(
            segment_id=SEGMENT_ID, host=HOST_ID, reserved=True)
        self._test_take_action(parsed_args)

    def test_take_action_by_name(self):

        # command param
        parsed_args = FakeNamespace(
            segment_id=SEGMENT_ID, host=HOST_NAME, reserved=True)
        self._test_take_action(parsed_args)

    @mock.patch.object(utils, 'get_dict_properties')
    def _test_take_action(self, parsed_args, mock_get_dict_properties):
        # return value segment list
        self.app.client_manager.ha.segments.return_value = self.dummy_segments
        # return value host list
        self.app.client_manager.ha.hosts.return_value = self.dummy_hosts
        # return value host show
        self.app.client_manager.ha.get_host.return_value = self.dummy_host
        # show the host specified by uuid
        self.update_host.take_action(parsed_args)
        self.app.client_manager.ha.update_host.assert_called_once_with(
            HOST_ID, segment_id=SEGMENT_ID, reserved=True)
        self.app.client_manager.ha.get_host.assert_called_once_with(
            HOST_ID, segment_id=SEGMENT_ID)
        mock_get_dict_properties.assert_called_once_with(
            self.dummy_host.to_dict(), self.columns, formatters={})


class TestV1DeleteHost(BaseV1Host):
    def setUp(self):
        super(TestV1DeleteHost, self).setUp()
        self.delete_host = DeleteHost(self.app, self.app_args,
                                      cmd_name='host delete')

    def test_take_action_by_uuid(self):

        # command param
        parsed_args = FakeNamespace(segment_id=SEGMENT_ID, host=HOST_ID)
        self._test_take_action(parsed_args)

    def test_take_action_by_name(self):

        # command param
        parsed_args = FakeNamespace(segment_id=SEGMENT_ID, host=HOST_NAME)
        self._test_take_action(parsed_args)

    def _test_take_action(self, parsed_args):
        # return value segment list
        self.app.client_manager.ha.segments.return_value = self.dummy_segments
        # return value host list
        self.app.client_manager.ha.hosts.return_value = self.dummy_hosts
        # return value host show
        self.app.client_manager.ha.delete_host.return_value = None
        # show the host specified by uuid
        self.delete_host.take_action(parsed_args)

        self.app.client_manager.ha.delete_host.assert_called_once_with(
            HOST_ID, segment_id=SEGMENT_ID, ignore_missing=False)



# Copyright(c) 2016 Nippon Telegraph and Telephone Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class BaseException(Exception):
    """An error occurred."""
    def __init__(self, message=None):
        self.message = message

    def __str__(self):
        return self.message or self.__class__.__doc__


class CommandError(BaseException):
    """Invalid usage of CLI."""


class UnsupportedVersion(Exception):
    """User is trying to use an unsupported version of the API."""
    pass



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

from masakariclient.common import exception
from masakariclient.common.i18n import _


_type_error_msg = _("'%(other)s' should be an instance of '%(cls)s'")


class APIVersion(object):
    """This class represents an API Version Request.

    This class provides convenience methods for manipulation
    and comparison of version numbers that we need to do to
    implement microversions.
    """

    def __init__(self, version_str=None):
        """Create an API version object.

        :param version_str: String representation of APIVersionRequest.
                            Correct format is 'X.Y', where 'X' and 'Y'
                            are int values. None value should be used
                            to create Null APIVersionRequest, which is
                            equal to 0.0
        """
        self.ver_major = 0
        self.ver_minor = 0

        if version_str is not None:
            match = re.match(r"^([1-9]\d*)\.([1-9]\d*|0|latest)$", version_str)
            if match:
                self.ver_major = int(match.group(1))
                if match.group(2) == "latest":
                    # Infinity allows to easily determine latest version and
                    # doesn't require any additional checks in comparison
                    # methods.
                    self.ver_minor = float("inf")
                else:
                    self.ver_minor = int(match.group(2))
            else:
                msg = _("Invalid format of client version '%s'. "
                        "Expected format 'X.Y', where X is a major part and Y "
                        "is a minor part of version.") % version_str
                raise exception.UnsupportedVersion(msg)

    def __str__(self):
        """Debug/Logging representation of object."""
        if self.is_latest():
            return "Latest API Version Major: %s" % self.ver_major
        return ("API Version Major: %s, Minor: %s"
                % (self.ver_major, self.ver_minor))

    def __repr__(self):
        if self.is_null():
            return "<APIVersion: null>"
        else:
            return "<APIVersion: %s>" % self.get_string()

    def is_null(self):
        return self.ver_major == 0 and self.ver_minor == 0

    def is_latest(self):
        return self.ver_minor == float("inf")

    def __lt__(self, other):
        if not isinstance(other, APIVersion):
            raise TypeError(_type_error_msg % {"other": other,
                                               "cls": self.__class__})

        return ((self.ver_major, self.ver_minor) <
                (other.ver_major, other.ver_minor))

    def __eq__(self, other):
        if not isinstance(other, APIVersion):
            raise TypeError(_type_error_msg % {"other": other,
                                               "cls": self.__class__})

        return ((self.ver_major, self.ver_minor) ==
                (other.ver_major, other.ver_minor))

    def __gt__(self, other):
        if not isinstance(other, APIVersion):
            raise TypeError(_type_error_msg % {"other": other,
                                               "cls": self.__class__})

        return ((self.ver_major, self.ver_minor) >
                (other.ver_major, other.ver_minor))

    def __le__(self, other):
        return self < other or self == other

    def __ne__(self, other):
        return not self.__eq__(other)

    def __ge__(self, other):
        return self > other or self == other

    def matches(self, min_version, max_version):
        """Matches the version object.

        Returns whether the version object represents a version
        greater than or equal to the minimum version and less than
        or equal to the maximum version.

        :param min_version: Minimum acceptable version.
        :param max_version: Maximum acceptable version.
        :returns: boolean

        If min_version is null then there is no minimum limit.
        If max_version is null then there is no maximum limit.
        If self is null then raise ValueError
        """

        if self.is_null():
            raise ValueError(_("Null APIVersion doesn't support 'matches'."))
        if max_version.is_null() and min_version.is_null():
            return True
        elif max_version.is_null():
            return min_version <= self
        elif min_version.is_null():
            return self <= max_version
        else:
            return min_version <= self <= max_version

    def get_string(self):
        """Version string representation.

        Converts object to string representation which if used to create
        an APIVersion object results in the same version.
        """
        if self.is_null():
            raise ValueError(
                _("Null APIVersion cannot be converted to string."))
        elif self.is_latest():
            return "%s.%s" % (self.ver_major, "latest")
        return "%s.%s" % (self.ver_major, self.ver_minor)



# Copyright(c) 2016 Nippon Telegraph and Telephone Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from openstack.connection import Connection
from osc_lib import utils

LOG = logging.getLogger(__name__)

DEFAULT_HA_API_VERSION = '1.3'
API_VERSION_OPTION = 'os_ha_api_version'
API_NAME = 'ha'

SUPPORTED_VERSIONS = [
    '1',
    '1.0',
    '1.1',
    '1.2',
    '1.3',
]

API_VERSIONS = {v: None
                for v in SUPPORTED_VERSIONS}


def make_client(instance):
    """Returns a instance_ha proxy"""
    LOG.debug('Instantiating masakari service client')
    con = Connection(session=instance.session,
                     interface=instance.interface,
                     region_name=instance.region_name,
                     ha_api_version=instance._api_version[API_NAME])
    return con.instance_ha


def build_option_parser(parser):
    """Hook to add global options"""
    parser.add_argument(
        '--os-ha-api-version',
        metavar='<ha-api-version>',
        default=utils.env(
            'OS_HA_API_VERSION',
            default=DEFAULT_HA_API_VERSION),
        help='ha API version, default=' +
             DEFAULT_HA_API_VERSION +
             ' (Env: OS_HA_API_VERSION)')
    return parser



# Copyright(c) 2019 NTT DATA
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
test_masakariclient
----------------------------------

Tests for `masakariclient` module.
"""
from unittest import mock
import uuid

from osc_lib.tests import utils as osc_lib_utils
from osc_lib import utils

from masakariclient.osc.v1.notification import ShowNotification
from masakariclient.tests import base

NOTIFICATION_NAME = 'notification_name'
NOTIFICATION_ID = uuid.uuid4()
RECOVERY_WORKFLOW_DETAILS = [{
    "progress": 1.0, "state": "SUCCESS",
    "name": "DisableComputeNodeTask",
    "progress_details": [
        {"timestamp:": "2019-02-28 07:21:33.170190",
         "progress": 0.5,
         "message": "Disabling compute host: host"},
        {"timestamp:": "2019-02-28 07:21:33.291810",
         "progress": 1.0,
         "message": "Skipping recovery for process nova-compute "
                    "as it is already disabled"}]}]


class FakeNotification(object):
    """Fake notification show detail."""
    def __init__(self,):
        super(FakeNotification, self).__init__()

    def to_dict(self):
        return {
            'created_at': '2019-02-18T05:47:46.000000',
            'updated_at': '2019-02-18T06:05:16.000000',
            'notification_uuid': NOTIFICATION_ID,
            'source_host_uuid': '9ab67dc7-110a-4a4c-af64-abc6e5798433',
            'name': NOTIFICATION_NAME,
            'id': 1,
            'type': 'VM',
            'payload': {
                "instance_uuid": "99ffc832-2252-4a9e-9b98-28bc70f7ff09",
                "vir_domain_event": "STOPPED_FAILED", "event": "LIFECYCLE"},
            'status': 'finished',
            'recovery_workflow_details': RECOVERY_WORKFLOW_DETAILS,
            'generated_time': '2019-02-13T15:34:55.000000'
            }


class BaseV1Notification(base.TestCase, osc_lib_utils.TestCommand):
    def setUp(self):
        super(BaseV1Notification, self).setUp()
        self.app = mock.Mock()
        self.app_args = mock.Mock()
        self.client_manager = mock.Mock()
        self.client_manager.default_microversion = '1.0'
        self.app.client_manager.ha = self.client_manager
        self.dummy_notification = FakeNotification()
        self.show_notification = ShowNotification(
            self.app, self.app_args, cmd_name='notification show')
        self.columns = ['created_at', 'updated_at', 'notification_uuid',
                        'type', 'status', 'source_host_uuid',
                        'generated_time', 'payload']


class TestShowNotificationV1(BaseV1Notification):

    def test_take_action_by_uuid(self):
        arglist = ['8c35987c-f416-46ca-be37-52f58fd8d294']
        parsed_args = self.check_parser(self.show_notification, arglist, [])
        self._test_take_action(parsed_args)

    @mock.patch.object(utils, 'get_dict_properties')
    def _test_take_action(self, parsed_args, mock_get_dict_properties):
        self.app.client_manager.ha.get_notification.return_value = (
            self.dummy_notification)

        self.show_notification.take_action(parsed_args)
        mock_get_dict_properties.assert_called_once_with(
            self.dummy_notification.to_dict(), self.columns, formatters={})


class TestShowNotificationV1_1(TestShowNotificationV1):

    def setUp(self):
        super(TestShowNotificationV1_1, self).setUp()
        self.client_manager.default_microversion = '1.1'
        self.columns.append('recovery_workflow_details')
